@@ -6,7 +6,6 @@
 ;;; Code:
 
 (require 'mew)
-(if mew-xemacs-p (require 'easymenu))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -47,10 +46,11 @@
     'mew-summary-analyze-again-alternative)
   (define-key mew-summary-mode-map ","    'mew-summary-display-asis)
   (define-key mew-summary-mode-map ";"    'mew-summary-trace-path)
-  (define-key mew-summary-mode-map "<"    'mew-summary-jump-top)
-  (define-key mew-summary-mode-map ">"    'mew-summary-jump-bottom)
-  (define-key mew-summary-mode-map "\177" 'mew-summary-prev-page)
-  (define-key mew-summary-mode-map [delete] 'mew-summary-prev-page)
+  (define-key mew-summary-mode-map "\e<"  'mew-summary-jump-top)
+  (define-key mew-summary-mode-map "\e>"  'mew-summary-jump-bottom)
+  (define-key mew-summary-mode-map "<"    'mew-summary-scroll-right)
+  (define-key mew-summary-mode-map ">"    'mew-summary-scroll-left)
+  (define-key mew-summary-mode-map "\d"   'mew-summary-prev-page)
   (define-key mew-summary-mode-map "\r"   'mew-summary-scroll-up)
   (define-key mew-summary-mode-map "-"    'mew-summary-scroll-down)
   (define-key mew-summary-mode-map "\e\r" 'mew-summary-scroll-down)
@@ -104,24 +104,26 @@
   (define-key mew-summary-mode-map "\C-c\t"   'mew-summary-find-file)
   (define-key mew-summary-mode-map "\C-c\C-f" 'mew-pgp-fetch-key)
   (define-key mew-summary-mode-map "\C-c\C-v" 'mew-pgp-select)
+  (define-key mew-summary-mode-map "\C-c\C-m" 'mew-passwd-change)
   (define-key mew-summary-mode-map "\C-c\C-s" 'mew-summary-isearch-forward)
   (define-key mew-summary-mode-map "\C-c\C-r" 'mew-summary-isearch-backward)
-  (define-key mew-summary-mode-map "\C-c\C-o" 
+  (define-key mew-summary-mode-map "\C-c\C-o"
     'mew-summary-jump-to-draft-buffer)
   (define-key mew-summary-mode-map "\ea"  'mew-summary-alias-edit)
   (define-key mew-summary-mode-map "\el"  'mew-summary-recenter)
   (define-key mew-summary-mode-map "\et"  'mew-summary-uudecode)
-  (define-key mew-summary-mode-map "\es"  'mew-summary-unshar)
+  (define-key mew-summary-mode-map "\es"  'mew-summary-sort)
   (define-key mew-summary-mode-map "\eb"  'mew-summary-burst-multi)
   (define-key mew-summary-mode-map "\e\\" 'mew-summary-cmd-msgs)
   (define-key mew-summary-mode-map "v"    'mew-summary-toggle-disp-msg)
+  (define-key mew-summary-mode-map "V"    'mew-summary-selection-by-msgid)
   (define-key mew-summary-mode-map "\C-c\C-l" 'mew-summary-convert-local-cs)
   (define-key mew-summary-mode-map "\C-c\C-z" 'mew-summary-decode-old-pgp)
   (define-key mew-summary-mode-map "\C-c\C-q" 'mew-summary-kill)
   (define-key mew-summary-mode-map "\C-c\C-x" 'mew-summary-x-face)
   (define-key mew-summary-mode-map "\C-c\C-k" 'mew-summary-kill-subprocess)
-  (define-key mew-summary-mode-map "^"    'mew-summary-thread-parent)
-  (define-key mew-summary-mode-map "&"    'mew-summary-thread-child)
+  (define-key mew-summary-mode-map "^"    'mew-summary-parent)
+  (define-key mew-summary-mode-map "&"    'mew-summary-child)
   (define-key mew-summary-mode-map "("    'mew-summary-thread-sibling-up)
   (define-key mew-summary-mode-map ")"    'mew-summary-thread-sibling-down)
   (define-key mew-summary-mode-map "m"	  (make-sparse-keymap))
@@ -140,17 +142,17 @@
   (define-key mew-summary-mode-map "ml"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "mlc"  'mew-summary-mark-local-copy)
   (define-key mew-summary-mode-map "mli"  'mew-summary-mark-imap-copy)
-  (define-key mew-summary-mode-map "V"    'mew-summary-virtual-old)
-  (define-key mew-summary-mode-map "/"    'mew-summary-virtual)
+  (define-key mew-summary-mode-map "/"    'mew-summary-selection-by-pick)
   (define-key mew-summary-mode-map "?"    'mew-summary-pick)
   (define-key mew-summary-mode-map "'"    'mew-summary-grep-old)
-  (define-key mew-summary-mode-map "m/"   'mew-summary-mark-virtual)
+  (define-key mew-summary-mode-map "m/"   'mew-summary-selection-by-mark)
   (define-key mew-summary-mode-map "mt"   'mew-summary-mark-thread)
   (define-key mew-summary-mode-map "t"    (make-sparse-keymap))
   (define-key mew-summary-mode-map "tt"   'mew-summary-make-thread)
   (define-key mew-summary-mode-map "tn"   'mew-thread-down)
   (define-key mew-summary-mode-map "tp"   'mew-thread-up)
   (define-key mew-summary-mode-map "t "   'mew-thread-toggle)
+  (define-key mew-summary-mode-map "ta"   'mew-thread-toggle-all)
   (define-key mew-summary-mode-map "t("   'mew-thread-all-graft)
   (define-key mew-summary-mode-map "t)"   'mew-thread-all-prune)
   (define-key mew-summary-mode-map "t*"   'mew-thread-mark-review)
@@ -178,12 +180,16 @@
   (define-key mew-summary-mode-map "lh"   'mew-summary-learn-ham)
   (define-key mew-summary-mode-map "ls"   'mew-summary-learn-spam)
   (define-key mew-summary-mode-map "="    'mew-summary-info)
+  (define-key mew-summary-mode-map "_"    'mew-summary-line)
   (define-key mew-summary-mode-map "k"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "kc"   'mew-summary-search-change-method)
   (define-key mew-summary-mode-map "k?"   'mew-summary-search)
-  (define-key mew-summary-mode-map "k/"   'mew-summary-search-virtual)
+  (define-key mew-summary-mode-map "k/"   'mew-summary-selection-by-search)
   (define-key mew-summary-mode-map "km"   'mew-summary-make-index-folder)
   (define-key mew-summary-mode-map "kM"   'mew-summary-make-index-all)
+  (define-key mew-summary-mode-map "ki"   'mew-summary-make-id-index-folder)
+  (define-key mew-summary-mode-map "kI"   'mew-summary-make-id-index-all)
+  (define-key mew-summary-mode-map "kj"   'mew-summary-goto-original-message)
   ;;
   ;; not provided in Virtual mode
   ;;
@@ -201,16 +207,14 @@
   (define-key mew-summary-mode-map "mxo"   'mew-summary-exec-refile)
   (define-key mew-summary-mode-map "s"    'mew-summary-ls)
   (define-key mew-summary-mode-map "O"    'mew-summary-pack)
-  (define-key mew-summary-mode-map "S"    'mew-summary-sort)
+  (define-key mew-summary-mode-map "S"    'mew-summary-selection-by-sort)
   (define-key mew-summary-mode-map "\C-c\C-b" 'mew-summary-exchange-point)
   (define-key mew-summary-mode-map "\eo"  'mew-summary-auto-refile)
   (define-key mew-summary-mode-map "R"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "Rd"   'mew-summary-delete-folder)
   (define-key mew-summary-mode-map "Rr"   'mew-summary-rename-folder)
   ;;
-  (if mew-xemacs-p
-      (define-key mew-summary-mode-map 'button2 'mew-summary-mouse-show)
-    (define-key mew-summary-mode-map [mouse-2] 'mew-summary-mouse-show)))
+  (define-key mew-summary-mode-map [mouse-2] 'mew-summary-mouse-show))
 
 (defvar mew-summary-mode-menu-spec
   '("Mew"
@@ -253,7 +257,7 @@
      )
     ("Select"
      ["Pick then mark"     mew-summary-pick        (mew-pickable-p)]
-     ["Virtual mode"       mew-summary-virtual     (mew-pickable-p)]
+     ["Selection"          mew-summary-selection-by-pick     (mew-pickable-p)]
      )
     ("Misc"
      ["Clean +trash"         mew-summary-clean-trash      t]
@@ -284,8 +288,7 @@
 (unless mew-message-mode-map
   (setq mew-message-mode-map (make-sparse-keymap))
   (define-key mew-message-mode-map " "    'mew-message-next-page)
-  (define-key mew-message-mode-map "\177" 'mew-message-prev-page)
-  (define-key mew-message-mode-map [delete] 'mew-message-prev-page)
+  (define-key mew-message-mode-map "\d"   'mew-message-prev-page)
   (define-key mew-message-mode-map "n"    'mew-message-next-msg)
   (define-key mew-message-mode-map "p"    'mew-message-prev-msg)
   (define-key mew-message-mode-map "h"    'mew-message-goto-summary)
@@ -293,9 +296,11 @@
   (define-key mew-message-mode-map "A"    'mew-message-reply-with-citation)
   (define-key mew-message-mode-map "f"    'mew-message-forward)
   (define-key mew-message-mode-map "r"    'mew-message-resend)
-  (define-key mew-message-mode-map "k"	  (make-sparse-keymap))
-  (define-key mew-message-mode-map "kp"   'mew-nmz-search-msgid-at-point)
-  (define-key mew-message-mode-map "kr"   'mew-nmz-search-msgid-region))
+  (define-key mew-message-mode-map "<"    'mew-message-scroll-right)
+  (define-key mew-message-mode-map ">"    'mew-message-scroll-left)
+  (define-key mew-message-mode-map "_"    'mew-message-line)
+  (define-key mew-message-mode-map "g"    'mew-browse-url-at-point)
+  (define-key mew-message-mode-map "\C-i" 'mew-message-goto-next-url))
 
 (defvar mew-message-mode-menu-spec
   '("Mew"
@@ -324,6 +329,8 @@
   (define-key (symbol-value symmap) "\C-c\C-e" 'mew-draft-encrypt-message)
   (define-key (symbol-value symmap) "\C-c\C-b" 'mew-draft-sign-encrypt-message)
   (define-key (symbol-value symmap) "\C-c\C-r" 'mew-draft-encrypt-sign-message)
+  (define-key (symbol-value symmap) "\C-c\C-p\C-m"
+    'mew-draft-set-privacy-method)
   (define-key (symbol-value symmap) "\C-c\C-p\C-a"
     'mew-draft-toggle-privacy-always)
   (define-key (symbol-value symmap) "\C-c\C-p\C-e"
@@ -341,7 +348,7 @@
 
 (unless mew-draft-body-map
   (setq mew-draft-body-map (make-sparse-keymap))
-  (mew-set-keymap-parent mew-draft-body-map text-mode-map)
+  (set-keymap-parent mew-draft-body-map text-mode-map)
   (define-key mew-draft-body-map "\C-c\t"       'mew-draft-insert-signature)
   (define-key mew-draft-body-map "\C-c\C-y"     'mew-draft-cite)
   (define-key mew-draft-body-map "\C-c\C-t"     'mew-draft-yank)
@@ -354,7 +361,7 @@
 
 (unless mew-draft-mode-map
   (setq mew-draft-mode-map (make-sparse-keymap))
-  (mew-set-keymap-parent mew-draft-mode-map mew-draft-body-map))
+  (set-keymap-parent mew-draft-mode-map mew-draft-body-map))
 
 (defvar mew-draft-mode-toolbar-menu
   '("Attachment Commands"
@@ -372,7 +379,7 @@
      (mew-attach-not-line012-1)]
     ["Insert a Sub-Multipart"
      mew-attach-multipart
-     (mew-attach-not-line012-1)] 
+     (mew-attach-not-line012-1)]
     ["Read a New File into a Buffer"
      mew-attach-find-new-file
      (mew-attach-not-line012-1)]
@@ -381,7 +388,7 @@
      (mew-attach-not-line012-1)]
     "----"
     ["Delete This Part"
-     mew-attach-delete 
+     mew-attach-delete
      (mew-attach-not-line012-1-dot)]
     "----"
     ["Describe This Part"
@@ -398,7 +405,7 @@
      (mew-attach-not-line0-1-dot)]
     ["Encode with Base64"
      mew-attach-base64
-     (mew-attach-not-line0-1-dot)]	 
+     (mew-attach-not-line0-1-dot)]	
     ["Encode with Quoted-Printable"
      mew-attach-quoted-printable
      (mew-attach-not-line0-1-dot)]
@@ -406,7 +413,7 @@
      mew-attach-pgp-sign
      (mew-attach-not-line0-1-dot)]
     ["Encrypt with PGP"
-     mew-attach-pgp-enc 
+     mew-attach-pgp-enc
      (mew-attach-not-line0-1-dot)]
     "----"
     ["Read This File into a Buffer"
@@ -498,6 +505,7 @@
   (define-key mew-draft-attach-map "S"      'mew-attach-pgp-sign)
   (define-key mew-draft-attach-map "E"      'mew-attach-pgp-enc)
   (define-key mew-draft-attach-map "\es"    'mew-attach-smime-sign)
+  (define-key mew-draft-attach-map "\ee"    'mew-attach-smime-enc)
   (define-key mew-draft-attach-map "p"      'mew-attach-pgp-public-key)
   (define-key mew-draft-attach-map "U"      'mew-attach-undo)
   (define-key mew-draft-attach-map "\C-c\t" 'mew-draft-insert-signature)
@@ -510,7 +518,7 @@
 
 (unless mew-addrbook-mode-map
   (setq mew-addrbook-mode-map (make-sparse-keymap))
-  (mew-set-keymap-parent mew-addrbook-mode-map text-mode-map)
+  (set-keymap-parent mew-addrbook-mode-map text-mode-map)
   (define-key mew-addrbook-mode-map "\C-c\C-c" 'mew-addrbook-register)
   (define-key mew-addrbook-mode-map "\C-c\C-q" 'mew-addrbook-kill))
 
@@ -526,11 +534,9 @@
 	  (make-sparse-keymap)))
   (define-key mew-input-map "\r"     'mew-input-exit-minibuffer)
   (define-key mew-input-map "\n"     'mew-input-exit-minibuffer)
-  (define-key mew-input-map [return] 'mew-input-exit-minibuffer)
   (define-key mew-input-map ","      'mew-input-comma)
   (define-key mew-input-map " "      'mew-input-complete)
   (define-key mew-input-map "\t"     'mew-input-complete)
-  (define-key mew-input-map [tab]    'mew-input-complete)
   (define-key mew-input-map "?"      'mew-input-complete)
   (define-key mew-input-map "\C-c\t" 'mew-circular-complete-switch)
   (define-key mew-input-map [menu-bar minibuf]
@@ -549,12 +555,10 @@
       (define-key mew-input-folder-map (char-to-string c)
 	'mew-input-folder-self-insert)
       (setq c (1+ c))))
-  (define-key mew-input-folder-map [delete]    'mew-input-folder-self-insert)
-  (define-key mew-input-folder-map [backspace] 'mew-input-folder-self-insert)
+  (define-key mew-input-folder-map "\d"     'mew-input-folder-self-insert)
   (define-key mew-input-folder-map "\C-g"   'mew-input-folder-abort-minibuffer)
   (define-key mew-input-folder-map "\r"     'mew-input-folder-exit-minibuffer)
   (define-key mew-input-folder-map "\n"     'mew-input-folder-exit-minibuffer)
-  (define-key mew-input-folder-map [return] 'mew-input-folder-exit-minibuffer)
   (define-key mew-input-folder-map "+"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "%"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "$"      'mew-input-folder-prefix)
@@ -607,13 +611,11 @@
 ;;;
 
 (defun mew-which-mew-etc ()
-  (let ((file "mew.el")
-	(path load-path))
+  (let ((file "mew.el"))
     (catch 'loop
-      (while path
-	(if (file-exists-p (expand-file-name file (car path)))
-	    (throw 'loop (expand-file-name "etc" (car path)))
-	  (setq path (cdr path)))))))
+      (dolist (path load-path)
+	(if (file-exists-p (expand-file-name file path))
+	    (throw 'loop (expand-file-name "etc" path)))))))
 
 (defvar mew-icon-directory (mew-which-mew-etc))
 
@@ -648,13 +650,13 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2005 Mew developing team.
+;; Copyright (C) 2000-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -663,7 +665,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
