@@ -34,7 +34,7 @@ This function are used to check validity of 'case' and sort key.")
                     (fboundp mew-input-exit-minibuffer-function)))
           (funcall mew-input-exit-minibuffer-function))
       (exit-minibuffer)))
- 
+
 (defvar mew-input-comma-function nil
   "A function to be called when ',' is typed in minibuffer.
 This function can be used to check validity of 'case'.")
@@ -107,10 +107,7 @@ at completions."
 	   (goto-char (match-beginning 1))
 	   (setq face (if (not window-system)
 			  'mew-face-header-from
-			(if (or (and mew-xemacs-p (find-face 'isearch))
-				(and (not mew-xemacs-p) (facep 'isearch)))
-			    'isearch
-			  'region)))
+			(if (facep 'isearch) 'isearch 'region)))
 	   (put-text-property (point) (match-end 1) 'face face)
 	   (unless (pos-visible-in-window-p (point) win)
 	     (set-window-start win (progn (forward-line -2) (point))))))))))
@@ -219,7 +216,7 @@ at completions."
 		   (save-excursion (end-of-line) (point)))))
 	(setq no-match t)
 	(goto-char pos)))
-    (if no-match 
+    (if no-match
 	(mew-input-folder-display "no match")
       (mew-input-folder-display))))
 
@@ -260,7 +257,7 @@ at completions."
 		   (save-excursion (end-of-line) (point)))))
 	(setq no-match t)
 	(goto-char pos)))
-    (if no-match 
+    (if no-match
 	(mew-input-folder-display "no match")
       (mew-input-folder-display))))
 
@@ -334,20 +331,12 @@ and does not insert anything on single folder input."
 When in folder search mode, this function searches a candidate
 folder and displays it in addition to its bound key."
   (interactive)
-  (let* ((key (if mew-xemacs-p
-		  (event-key last-command-event)
-		(this-command-keys)))
-	 last-str gfunc)
+  (let ((key (this-command-keys))
+	last-str gfunc)
     (cond
-     ((mew-characterp key) ;; XEmacs
-      (setq key (char-to-string key))
-      (setq last-str key)
-      (setq gfunc (lookup-key (current-global-map) key)))
      ((stringp key)
       (setq last-str key)
       (setq gfunc (lookup-key (current-global-map) key)))
-     ((symbolp key) ;; XEmacs
-      (setq gfunc (lookup-key (current-global-map) key)))
      ((vectorp key)
       (setq gfunc (lookup-key (current-global-map) key))
       (unless gfunc
@@ -409,7 +398,7 @@ folder and displays it in addition to its bound key."
 
 (defun mew-input-folder-prefix ()
   "A function to insert a folder prefix in minibuffer.
-If the previous character is another folder prefix, 
+If the previous character is another folder prefix,
 it is deleted automatically."
   (interactive)
   (cond
@@ -429,10 +418,10 @@ it is deleted automatically."
 	    (save-excursion (forward-char -1) (looking-at ",")))
 	(if insert-ok (insert prefix)))
        ((or (and (= pos 1)
-		 (save-excursion (forward-char -1) (looking-at "[-+%$]"))) 
+		 (save-excursion (forward-char -1) (looking-at "[-+%$]")))
 	    ;; excluding *
 	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
-	;; A wrong prefix might accidentally be here. 
+	;; A wrong prefix might accidentally be here.
 	;; So, replace it just in case
 	(when insert-ok
 	  (forward-char -1)
@@ -449,7 +438,7 @@ it is deleted automatically."
 	(insert last-command-char))
        ;;
        ((or (and (= pos 1)
-		 (save-excursion (forward-char -1) (looking-at "[-+%$]"))) 
+		 (save-excursion (forward-char -1) (looking-at "[-+%$]")))
 	    ;; excluding *
 	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
 	(forward-char -1)
@@ -479,10 +468,8 @@ it is deleted automatically."
 ;;;
 
 (defun mew-input-folder-check (folders)
-  (let (case:folder folder ret)
-    (while folders
-      (setq case:folder (car folders))
-      (setq folders (cdr folders))
+  (let (folder ret)
+    (dolist (case:folder folders)
       (catch 'continue
 	(if (string-match mew-regex-case case:folder)
 	    (progn
@@ -495,9 +482,6 @@ it is deleted automatically."
 	(if (and (mew-folder-popp folder) ;; not $inbox
 		 (not (string= mew-pop-inbox-folder folder)))
 	    (throw 'continue nil))
-	(when (string-match " " folder)
-	  (mew-warn "SPC is not allowed!")
-	  (throw 'continue nil))
 	(setq case:folder (mew-canonicalize-case-folder case:folder))
 	(setq ret (cons case:folder ret))))
     (setq ret (delete nil ret))
@@ -529,10 +513,20 @@ it is deleted automatically."
 	 ;; mew-inherit-case must be nil
 	 (ret (read-from-minibuffer (format "Folder name (%s): " default)
 				    init mew-input-folder-map nil
-				    'mew-input-folder-hist)))
+				    'mew-input-folder-hist))
+	 (new-folder))
     (when (or (string= ret "") (string= ret init))
       (setq ret default))
-    (car (mew-input-folder-check (list ret)))))
+    ;; mew-input-folder-check requires an IMAP separator, 
+    ;; which requires C-uZ. C-uZ needs "g". So, chicken and egg.
+    ;; Skip the check for inbox to avoid the problem.
+    (setq new-folder (mew-case:folder-folder ret))
+    (if (member new-folder (list mew-inbox-folder
+				 mew-pop-inbox-folder
+				 mew-imap-inbox-folder
+				 mew-nntp-newsgroup))
+	ret
+      (car (mew-input-folder-check (list ret))))))
 
 (defun mew-input-folders (case:folder)
   "Input a folder from the minibuffer."
@@ -552,10 +546,8 @@ it is deleted automatically."
     (mew-input-folder-check ret)))
 
 (defun mew-input-refile-folder-check (folders proto)
-  (let (case:folder folder ret)
-    (while folders
-      (setq case:folder (car folders))
-      (setq folders (cdr folders))
+  (let (folder ret)
+    (dolist (case:folder folders)
       (catch 'continue
 	;; If case is specified, just remove it at this moment...
 	(if (string-match mew-regex-case case:folder)
@@ -573,9 +565,6 @@ it is deleted automatically."
 	(if (and (= (length folder) 1)
 		 (member folder mew-folder-prefixes)) ;; "prefix" only
 	    (throw 'continue nil))
-	(when (string-match " " folder)
-	  (mew-warn "SPC is not allowed!")
-	  (throw 'continue nil))
 	(setq folder (mew-canonicalize-case-folder folder))
 	(setq ret (cons folder ret))))
     (nreverse ret)))
@@ -626,7 +615,7 @@ it is deleted automatically."
   (mew-input-clear)
   (let ((mew-input-complete-function 'mew-complete-address)
 	val vals addrs ret)
-    (setq val (read-from-minibuffer 
+    (setq val (read-from-minibuffer
 	       (if default (format prompt default) prompt)
 	       ""
 	       mew-input-map
@@ -635,9 +624,7 @@ it is deleted automatically."
     (if (and default (string= val ""))
 	(setq val default))
     (setq vals (mapcar 'mew-chop (mew-split-quoted val ?,)))
-    (while vals
-      (setq val (car vals))
-      (setq vals (cdr vals))
+    (dolist (val vals)
       (setq addrs (mew-alias-expand val mew-addrbook-alist 0))
       (setq addrs (mapcar 'mew-addrstr-append-domain addrs))
       (setq ret (nconc ret addrs)))
@@ -719,16 +706,16 @@ it is deleted automatically."
 		      (format "Sort by (%s)?: " key)
 		      ""
 		      mew-input-map
-		      nil 
+		      nil
 		      'mew-input-sort-key-hist))
 	 field mode)
     (if (or (null field:mode) (string= field:mode ""))
 	(setq field:mode key))
     (setq field (car (mew-split field:mode ?:)))
-    (setq mode  (or (car (cdr (mew-split field:mode ?:)))
-		    (cdr (assoc field mew-sort-key-alist))
-		    "text"))
-    (cons field mode)))
+    (setq mode (or (car (cdr (mew-split field:mode ?:)))
+		   (mew-alist-get-value (assoc field mew-sort-key-alist))
+		   "text"))
+    (list field mode)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -876,7 +863,11 @@ it is deleted automatically."
        (setq def (concat dir ,file)))
      (setq ret (mew-input-file-name ,prompt def))
      (if ,preservep
-	 (setq ,previous (file-name-directory (mew-convert-to-home-dir ret))))
+	 (setq ,previous (file-name-directory
+			  (mew-convert-to-home-dir
+			   (if (file-directory-p ret)
+			       (file-name-as-directory ret)
+			     ret)))))
      (if (and ,file (file-directory-p ret))
 	 (setq ret (expand-file-name (file-name-nondirectory ,file) ret)))
      ret))
@@ -944,14 +935,12 @@ it is deleted automatically."
 ;;;
 
 (defun mew-input-case-check ()
-  (let* ((case (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
-         (lst (mew-split case ?,))
-	 err)
+  (let ((case (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
+	err)
     (catch 'nomatch
-      (while lst
-	(unless (member (car lst) mew-config-cases)
-	  (throw 'nomatch (setq err (car lst))))
-	(setq lst (cdr lst))))
+      (dolist (cs (mew-split case ?,))
+	(unless (member cs mew-config-cases)
+	  (throw 'nomatch (setq err cs)))))
     (if err
         (progn
           (mew-temp-minibuffer-message (format " [No match: %s]" err))
@@ -967,7 +956,7 @@ it is deleted automatically."
 	(mew-circular-complete-function 'mew-circular-complete-case)
 	(mew-input-exit-minibuffer-function 'mew-input-case-check)
 	(mew-input-comma-function 'mew-input-case-check)
-	case lst ret)
+	case ret)
     (if edit
 	(setq case (read-from-minibuffer
 		    "Case value: "
@@ -983,11 +972,9 @@ it is deleted automatically."
                   'mew-input-case-hist)))
     (if (string= case "")
 	default
-      (setq lst (mew-split case ?,))
-      (while lst
-	(if (member (car lst) mew-config-cases)
-	    (setq ret (cons (car lst) ret)))
-	(setq lst (cdr lst)))
+      (dolist (cs (mew-split case ?,))
+	(if (member cs mew-config-cases)
+	    (setq ret (cons cs ret))))
       (mapconcat 'identity (nreverse ret) ","))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1010,97 +997,6 @@ it is deleted automatically."
      (t (message "Mark %c is not supported" char)
 	nil))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Password
-;;;
-
-(defvar mew-passwd-alist nil)
-(defvar mew-passwd-timer-id nil)
-
-(defsubst mew-passwd-get-passwd (key)
-  (nth 1 (assoc key mew-passwd-alist)))
-(defsubst mew-passwd-get-counter (key)
-  (nth 2 (assoc key mew-passwd-alist)))
-
-(defun mew-passwd-set-passwd (key val)
-  (if (assoc key mew-passwd-alist)
-      (setcar (nthcdr 1 (assoc key mew-passwd-alist)) val)
-    (setq mew-passwd-alist (cons (list key val 0) mew-passwd-alist))))
-(defun mew-passwd-set-counter (key val)
-  (if (assoc key mew-passwd-alist)
-      (setcar (nthcdr 2 (assoc key mew-passwd-alist)) val)))
-
-(defun mew-passwd-get-keys ()
-  (mapcar 'car mew-passwd-alist))
-
-(defsubst mew-passwd-reset ()
-  (setq mew-passwd-alist nil))
-
-(defun mew-passwd-setup ()
-  (if mew-passwd-timer-id (cancel-timer mew-passwd-timer-id))
-  (setq mew-passwd-timer-id
-	(mew-timer (* mew-passwd-timer-unit 60) 'mew-passwd-timer)))
-
-(defun mew-passwd-clean-up ()
-  (mew-passwd-reset)
-  (if mew-passwd-timer-id (cancel-timer mew-passwd-timer-id))
-  (setq mew-passwd-timer-id nil))
-
-(defun mew-passwd-timer ()
-  (let ((keys (mew-passwd-get-keys)) key)
-    (while keys
-      (setq key (car keys))
-      (setq keys (cdr keys))
-      (if (< (mew-passwd-get-counter key) mew-passwd-lifetime)
-	  (mew-passwd-set-counter key (1+ (mew-passwd-get-counter key)))
-	;; time out
-	(mew-passwd-set-passwd key nil)
-	(mew-passwd-set-counter key 0)))))
-
-(defun mew-input-passwd (prompt key)
-  (if (and key mew-use-cached-passwd)
-      (if (mew-passwd-get-passwd key)
-	  (progn
-	    (mew-timing)
-	    (if mew-passwd-reset-timer (mew-passwd-set-counter key 0))
-	    (mew-passwd-get-passwd key))
-	(let ((pass (mew-read-passwd prompt)))
-	  (mew-passwd-set-passwd key pass)
-	  (mew-passwd-set-counter key 0)
-	  pass))
-    (mew-read-passwd prompt)))
-
-(defun mew-read-passwd (prompt)
-  (let ((inhibit-input-event-recording t)
-	;; A process filter sets inhibit-quit to t to prevent quitting.
-	;; Set inhibit-quit to nil so that C-g can be used
-	(inhibit-quit nil))
-    (condition-case nil
-	(read-passwd prompt)
-      ;; If read-passwd causes an error, let's return "" so that
-      ;; the password process will safely fail.
-      (quit "")
-      (error ""))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Language name
-;;;
-
-;; read-language-name should be used. Unfortunately, that of
-;; XEmacs 21.1.14 is broken. This function should be obsoleted
-;; when Mew quit support of XEmacs 21.1.14
-
-(defun mew-input-language-name (prompt &optional default) 
-  (let* ((completion-ignore-case t)
-         (name (completing-read prompt
-                                language-info-alist
-                                nil
-                                t nil nil)))
-    (if (string= name "") (setq name default))
-    name))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Encoding
@@ -1109,13 +1005,13 @@ it is deleted automatically."
 (defun mew-input-encoding ()
   (let* ((encoding-list (list mew-b64 mew-qp))
 	 (encoding-alist (mapcar (lambda (x) (list x)) encoding-list))
-	 (prompt (format "Lines are too long. Input encoding (%s): " 
+	 (prompt (format "Lines are too long. Input encoding (%s): "
 			 mew-default-encoding))
 	 (cte (completing-read prompt encoding-alist)))
     (if (string= cte "") (setq cte mew-default-encoding))
     (setq cte (downcase cte))
     (while (not (member cte encoding-list))
-      (setq prompt (format "'%s' is unknown. Input encoding (%s): " 
+      (setq prompt (format "'%s' is unknown. Input encoding (%s): "
 			   cte mew-default-encoding))
       (setq cte (completing-read prompt encoding-alist))
       (if (string= cte "") (setq cte mew-default-encoding))
@@ -1141,13 +1037,13 @@ it is deleted automatically."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1156,7 +1052,7 @@ it is deleted automatically."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
